/*****************************************************************************/
/*                                                                           */
/*  QaSClassStatus                                                           */
/*                                                                           */
/*                                                                           */
/*     Description:                                                          */
/*             The table containing information about DiffServ classes.      */
/*                                                                           */
/*     History:                                                              */
/*             31/01/2003 - Fabien Chanussot - Creation                      */
/*                                                                           */
/*****************************************************************************/

//###FC - TBD package QaS;

/*****************************************************************************/
/* Use                                                                       */
/*****************************************************************************/

import java.util.*;
import java.io.*;


/*****************************************************************************/
/* QaSClassEntry class:                                                      */
/*                         Status information about a DiffServ class.        */
/*****************************************************************************/
class QaSClassEntry
{
    /*************************************************************************/
    /* Private                                                               */
    /*************************************************************************/
    
    
    /*************************************************************************/
    /* Public                                                                */
    /*************************************************************************/
    
    //The network dimensioning of a DiffServ class
    public int configured;
    
    //The amount of reserved bandwidth for that class
    public int allocated;
    
    //The amount of used bandwidth for the class (might be different from allocated: depends on commits) 
    public int used;
    
    
    /*
     * observeHTML(): built an HTML status page
     */
    public void observeHTML(StringBuffer pString, int i)
    {
	pString.append("<tr><td>");
	pString.append(i);
	pString.append("</td><td>");
	pString.append(this.configured);
	pString.append("</td><td>");
	pString.append(this.allocated);
	pString.append("</td></tr>");
    }
}


/*****************************************************************************/
/* QaSClassStatus class:                                                     */
/*                         Status information table.                         */
/*****************************************************************************/
class QaSClassStatus
{
    /*************************************************************************/
    /* Private                                                               */
    /*************************************************************************/
    
    //Maximum number of classes. The DSCP is coded on a 6-bits value
    protected static final int QAS_MAX_NUM_CLASSES = 256; //###FC - Fixe that:-(((
    
    
    /*************************************************************************/
    /* Public                                                                */
    /*************************************************************************/

    public QaSClassEntry myClassTable[];
    
    
    /*
     * Constructor:
     */
    public QaSClassStatus()
    {
	int i;
	
	myClassTable = new QaSClassEntry[QAS_MAX_NUM_CLASSES];
	
	for ( i = 0; i < QAS_MAX_NUM_CLASSES; i++ )
	    myClassTable[i] = new QaSClassEntry();
    }
    
    
    /*
     * dump:
     */
    public void dump()
    {
	int i;
	
	System.out.println("Dumping Class table:");
	
	for ( i = 0; i < QAS_MAX_NUM_CLASSES; i++ )
	    {
		if ( myClassTable[i].configured > myClassTable[i].allocated )
		    System.out.println("DSCP: " + i + ", configured: " + myClassTable[i].configured + ", allocated: " + myClassTable[i].allocated);
	    }
    }
    
    
    /*
     * observeHTML(): built an HTML status page
     */
    public void observeHTML(StringBuffer pString)
    {
	int i;
	
	//Add header
	pString.append("<p><bold>DiffServ classes status:</bold><br>");

	//Add table
	pString.append("<table border=\"2\"><tbody><tr><td><italic>Name</italic></td><td><italic>Configured</italic></td><td><italic>Allocated</italic></td></tr>");

	for ( i = 0; i < QAS_MAX_NUM_CLASSES; i++ )
	    {
		if ( myClassTable[i].configured > myClassTable[i].allocated )
		    myClassTable[i].observeHTML(pString, i);
	    }
	
	pString.append("</tbody></table>");
	
	//Add footer
	pString.append("</p>");
    }
}





