/*****************************************************************************/
/*                                                                           */
/*  QaSKernelIf                                                              */
/*                                                                           */
/*                                                                           */
/*     Description:                                                          */
/*             Object that interfaces with the kernel's marking and          */
/*             shaping module.                                               */
/*                                                                           */
/*     History:                                                              */
/*             17/02/2003 - Fabien Chanussot - Creation                      */
/*                                                                           */
/*****************************************************************************/

//###FC - TBD package QaS;

/*****************************************************************************/
/* Use                                                                       */
/*****************************************************************************/

import java.util.*;
import java.io.*;


/*****************************************************************************/
/* QaSKernelParam class:                                                     */
/*                                                                           */
/*****************************************************************************/
class QaSKernelParam
{
    /*************************************************************************/
    /* Public                                                                */
    /*************************************************************************/

    int local;
    int diffserv;
    int bucketrate;
    int activity;
    
    /*
     * Constructor
     */
    public QaSKernelParam(int local, int diffserv, int rate, int activity)
    {
	this.local = local;
	this.diffserv = diffserv;
	this.bucketrate = rate;
	this.activity = activity;
    }
    
    public QaSKernelParam()
    {
	this.local = 0;
	this.diffserv = 0;
	this.bucketrate = 0;
	this.activity = 0;
    }
    
}


/*****************************************************************************/
/* QaSKernelIf class:                                                        */
/*                                                                           */
/*****************************************************************************/
class QaSKernelIf
{
    /*************************************************************************/
    /* Private                                                               */
    /*************************************************************************/
    
    static final String QAS_DEVICE = "/dev/qasdev";
    static final int QAS_SIZEOF_KERNEL_SOFT_STATE = 8;
    
    /*************************************************************************/
    /* Public                                                                */
    /*************************************************************************/
    
    /*
     * setNetfilter():
     */
    static int setNetfilter(QaSKernelParam param)
    {
	RandomAccessFile fout;
	
	
	try
	    {
		fout = new RandomAccessFile(QAS_DEVICE, "rw");
		
		if ( fout != null )
		    {
			//seek to the correct position
			fout.seek(param.local * QAS_SIZEOF_KERNEL_SOFT_STATE);
			
			//pass the "activity" flag used by the soft state
			fout.writeInt(param.activity);			
			
			//pass the diffserv class to use for that stream (needs to be passed last, because of char alignment)
			fout.writeInt(param.diffserv);
			
			fout.close();
		    }
		
	    } catch (FileNotFoundException e) 
		{
		    System.err.println("QoSINUS! File not found! Is the module up and running?");
		    return(-1);
		}
	    catch (IOException e) 
		{
		    System.err.println("QoSINUS! setNetfilter " + e);
		    return(-1);
		}
	
	
	return(0);
    }
    
    
    /*
     * getNetfilter():
     */
    static int getNetfilter(QaSKernelParam param)
    {
	RandomAccessFile fin;
	
	
	try
	  {
		fin = new RandomAccessFile(QAS_DEVICE, "r");
		
		if ( fin != null )
		    {
			//seek to the correct position
			fin.seek(param.local * QAS_SIZEOF_KERNEL_SOFT_STATE);
			
			//pass the "activity" flag used by the soft state
			param.activity = fin.readInt();			
			
			//pass the diffserv class to use for that stream (needs to be passed last, because of char alignment)
			param.diffserv = fin.readInt();
			
			fin.close();
		    }
		
	    } catch (FileNotFoundException e) 
		{
		    System.err.println("QoSINUS! File not found! Is the module up and running?");
		    return(-1);
		}
	    catch (IOException e)
		{
		    System.err.println("QoSINUS! getNetfilter " + e);
		    return(-1);
		}
	
	
	return(0);
    
    }
  
    /*
     * setTc():
     */
    static int setTc(QaSKernelParam param, String pCommand)
    {
	String aCommand;
	
	aCommand = pCommand + Integer.toString(param.local) + " htb rate " + Integer.toString(param.bucketrate) + "mbps ceil " + Integer.toString(param.bucketrate) + "mbps";
	
	System.out.println("TC command: " + aCommand);	
	
	try
	    {
		Runtime.getRuntime().exec(aCommand);
	    }
	catch (IOException e)
	    {
		System.err.println("QoSINUS! setTc " + e);
		return(-1);
	    }
	
	
	return(0);
    }
}

