/*****************************************************************************/
/*                                                                           */
/*  QaSMonitoringIf                                                          */
/*                                                                           */
/*                                                                           */
/*     Description:                                                          */
/*             The DiffServ classes monitoring interface.                    */
/*                                                                           */
/*     History:                                                              */
/*             14/04/2003 - Fabien Chanussot - Creation                      */
/*                                                                           */
/*****************************************************************************/

//###FC - TBD package QaS;

/*****************************************************************************/
/* Use                                                                       */
/*****************************************************************************/

import java.util.*;
import java.io.*;
import java.net.*;


/*****************************************************************************/
/* QaSMonitoringIf class:                                                    */
/*                        The DiffServ classes monitoring client.            */
/*****************************************************************************/
public class QaSMonitoringIf
{
    /*************************************************************************/
    /* Protected                                                             */
    /*************************************************************************/
    
    Socket mySocket;
    String mySrvIp;
    int myPort;
    DataOutputStream mySout;
    DataInputStream mySin;
    
    
    /*
     * connect():
     */
    protected void connect()
    {
	try
	    {
		//Create socket
		mySocket = new Socket(mySrvIp, myPort);
		
		//Create streams to write to and read from the socket
		mySin = new DataInputStream(mySocket.getInputStream());
		mySout = new DataOutputStream(mySocket.getOutputStream());
		
		//System.out.println("### Connected to monitoring server.");
	    }
	catch (IOException e)
	    {
		System.err.println("Error connecting to monitoring server: " + e);
	    }
    }
    
    
    /*
     * disconnect():
     */
    protected void disconnect()
    {
	try
	    {
		//Close the socket...
		mySocket.close();
	    }
	catch (IOException e)
	    {
		System.err.println("Error closing connection to monitoring server: " + e);
	    }
    }
    
    
    /*************************************************************************/
    /* Public                                                                */
    /*************************************************************************/
    
    /*
     * Constructors
     */
    public QaSMonitoringIf(String ipSrv, int port)
    {
	mySrvIp = ipSrv;
	myPort = port;
    }
    
    
    /*
     * startClassMonitoring(): launch a remote monitoring thread
     */
    public void startClassMonitoring(int aClass, String ipAddress)
    {
	float aFloat;
	int anInt;
	
	try 
	    {
		connect();
		
		mySout.writeInt(1); //###FC - TBD: 1 = start monitoring. Make that cleaner.
		mySout.writeInt(aClass);
		//###FC - TBD mySout.writeInt(aTool);
		mySout.writeBytes(ipAddress);
		mySout.writeChar('\n'); //New line
		System.out.println("start command sent");
		
		anInt = mySin.readInt(); //Sync client and server
		
		disconnect();
	    }
	catch (IOException e)
	    {
		System.err.println("Error sending monitoring request: " + e);
	    }
    }
    
    
    /*
     * stopClassMonitoring(): stop a remote monitoring thread
     */
    public void stopClassMonitoring(int aClass, String ipAddress)
    {
	float aFloat;
	int anInt;
	
	try 
	    {
		connect();
		
		mySout.writeInt(2); //###FC - TBD: 1 = start monitoring. Make that cleaner.
		mySout.writeInt(aClass);
		//###FC - TBD mySout.writeInt(aTool);
		mySout.writeBytes(ipAddress);
		mySout.writeChar('\n'); //New line
		System.out.println("stop command sent");
		
		anInt = mySin.readInt(); //Sync client and server
		
		disconnect();
	    }
	catch (IOException e)
	    {
		System.err.println("Error sending monitoring request: " + e);
	    }
    }
    
    
    /*
     * getClassPerformance(): get monitoring result from remote thread
     */
    public float getClassPerformance(int aClass, String ipAddress)
    {
	float aFloat;
	
	try 
	    {
		connect();
		
		mySout.writeInt(3); //###FC - TBD: 3 = get monitoring. Make that cleaner.
		mySout.writeInt(aClass);
		//###FC - TBD mySout.writeInt(aTool);
		mySout.writeBytes(ipAddress);
		mySout.writeChar('\n'); //New line
		//System.out.println("get command sent");
		
		aFloat = mySin.readFloat();
		//System.out.println("float read : " + aFloat);
		
		disconnect();
	    }
	catch (IOException e)
	    {
		System.err.println("Error getting monitoring information: " + e);
		return(-1);
	    }
	
	return(aFloat);
    }
    
    
    /*
     * main(): pues eso, main
     */
    public static void main(String args[]) 
    {
	QaSMonitoringIf theMonitoringIf;
	float monvalue;
	
	//Create monitoring interface
	theMonitoringIf = new QaSMonitoringIf(args[0], 2001);
	
	//Start remote monitoring thread
	//	if ( args[2].equals("start") )
	    theMonitoringIf.startClassMonitoring(0, args[1]);
	
	//Retreive monitoring information
	    //if ( args[2].equals("get") )
	    monvalue = theMonitoringIf.getClassPerformance(0, args[1]);
	
	//Stop monitoring thread
	    //if ( args[2].equals("stop") )
	    theMonitoringIf.stopClassMonitoring(0, args[1]);
    }
}
