/*****************************************************************************/
/*                                                                           */
/*  QaSXMLIf                                                                 */
/*                                                                           */
/*                                                                           */
/*     Description:                                                          */
/*             Interface between QaS and the Xerces-J XML parser.            */
/*                                                                           */
/*     History:                                                              */
/*             28/01/2003 - Fabien Chanussot - Creation                      */
/*                                                                           */
/*****************************************************************************/

//###FC - TBD package QaS;

/*****************************************************************************/
/* Use                                                                       */
/*****************************************************************************/

import java.util.*;
import java.io.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Attr;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.InputSource;

import org.apache.xerces.dom.TextImpl;
import org.apache.xerces.parsers.DOMParser;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.Serializer;
import org.apache.xml.serialize.SerializerFactory;
import org.apache.xml.serialize.XMLSerializer;


/*****************************************************************************/
/* QaSXMLDoc class:                                                          */
/*                  An XML Document.                                         */
/*****************************************************************************/
class QaSXMLDoc
{
    /*************************************************************************/
    /* Private                                                               */
    /*************************************************************************/
    
    Document myDocument; //Dom document
    
    
    /*************************************************************************/
    /* Public                                                                */
    /*************************************************************************/

    /*
     * Constructor: 
     */
    public QaSXMLDoc(Document pDomDoc)
    {
	myDocument = pDomDoc;
	return;
    }
    
    
    /*
     * getAttribute(): extracts the value of the attribute pointed to by the given path
     */
    public String getAttribute(String pPath)
    {
	StringTokenizer aTokenizer;
	Element anElement;
	NodeList someElements;
	String aString;
	
	
	//Initialization
	aTokenizer = new StringTokenizer(pPath, "/");
	
	if ( aTokenizer.countTokens() == 0 )
	    return("");
	
	anElement = myDocument.getDocumentElement();
	aString = aTokenizer.nextToken();
	
	//Traverse the tree
	while ( aTokenizer.hasMoreTokens() )
	    {
		someElements = anElement.getElementsByTagName(aString);
		
		if ( someElements.getLength() != 1 )
		    return(""); //The path is wrong
		
		anElement = (Element)someElements.item(0);
		
		aString = aTokenizer.nextToken();
	    }
	
	//Get the attribute value
	return(anElement.getAttribute(aString));
    }
    
    
    /*
     * getFilteredAttribute(): extracts the value of the attribute pointed to by the given path, using another attribute to select the leaf element
     */
    public String getFilteredAttribute(String pPath, String pFilterName, String pFilterValue, String pAttrName)
    {
	StringTokenizer aTokenizer;
	Element anElement;
	NodeList someElements;
	String aString;
	int i;
	
	
	//Initialization
	aTokenizer = new StringTokenizer(pPath, "/");
	
	if ( aTokenizer.countTokens() == 0 )
	    {
		System.err.println("QoSINUS! Wrong pass filtering attributes");
		return("");
	    }
	
	anElement = myDocument.getDocumentElement();
	aString = aTokenizer.nextToken();
	
	//Traverse the tree
	while ( aTokenizer.hasMoreTokens() )
	    {
		someElements = anElement.getElementsByTagName(aString);
		
		if ( someElements.getLength() != 1 )
		    {
			System.err.println("QoSINUS! Wrong pass bis filtering attributes");
			return(""); //The path is wrong
		    }
		
		anElement = (Element)someElements.item(0);
		
		aString = aTokenizer.nextToken();
	    }
	
	//List elements to filter
	someElements = anElement.getElementsByTagName(aString);
	
	for ( i = someElements.getLength(); i > 0; i-- )
	    {
		anElement = (Element)someElements.item(i - 1);
		
		//###FC - DEBUG System.out.println("Filtering: filter: " + pFilterName + ", conf value: " + anElement.getAttribute(pFilterName) + ", compared to: " + pFilterValue);
		
		if ( anElement.getAttribute(pFilterName).compareTo(pFilterValue) == 0 )
		    return(anElement.getAttribute(pAttrName));
	    }
	
	//Couldn't find a match
	return("");
    }
    
    
    /*
     * setAttribute(): sets the value of the attribute pointed to by the given path
     */
    public void setAttribute(String pPath, String pValue)
    {
	StringTokenizer aTokenizer;
	Element anElement;
	NodeList someElements;
	String aString;
	
	
	//Initialization
	aTokenizer = new StringTokenizer(pPath, "/");
	
	if ( aTokenizer.countTokens() == 0 )
	    return;
	
	anElement = myDocument.getDocumentElement();
	aString = aTokenizer.nextToken();
	
	//Traverse the tree
	while ( aTokenizer.hasMoreTokens() )
	    {
		someElements = anElement.getElementsByTagName(aString);
		
		if ( someElements.getLength() != 1 )
		    return; //The path is wrong
		
		anElement = (Element)someElements.item(0);
		
		aString = aTokenizer.nextToken();
	    }
	
	//Set the attribute value
	anElement.setAttribute(aString, pValue);
    }
    
    
    /*
     * setAttributeCreate(): sets the value of the attribute pointed to by the given path, creating elements where needed
     */
    public void setAttributeCreate(String pPath, String pValue)
    {
	StringTokenizer aTokenizer;
	Element anElement;
	NodeList someElements;
	String aString;
	
	
	//Initialization
	aTokenizer = new StringTokenizer(pPath, "/");
	
	if ( aTokenizer.countTokens() == 0 )
	    return;
	
	anElement = myDocument.getDocumentElement();
	aString = aTokenizer.nextToken();
	
	//Traverse the tree
	while ( aTokenizer.hasMoreTokens() )
	    {
		someElements = anElement.getElementsByTagName(aString);
		
		if ( someElements.getLength() == 1 )
		    { //The path exists
		    anElement = (Element)someElements.item(0);
		    }
		else if ( someElements.getLength() == 0 )
		    { //The path needs to be created
			Element aNewElement = myDocument.createElement(aString);
			anElement.appendChild(aNewElement);
			anElement = aNewElement;
		    }
		else
		    return; //The path is wrong
		
		aString = aTokenizer.nextToken();
	    }
	
	//Set the attribute value
	anElement.setAttribute(aString, pValue);
    }
    
    
    /*
     * replaceAttributeByValueInElement(): change the value of the attributes of a given element
     */
    public void replaceAttributeByValueInElement(Element pElement, String pOldValue, String pNewValue)
    {
	int i;
	NamedNodeMap aNamedNodeMap;
	
	//###FC - DEBUG System.out.println("RE: old: " + pOldValue + " New: " + pNewValue);
	
	//List all the attributes
	aNamedNodeMap = pElement.getAttributes();

	//Check the value of each attribute of the element, and change it if needed
	for ( i = aNamedNodeMap.getLength(); i > 0; i-- )
	    {
		Attr anAttribute;
		
		//###FC - DEBUG System.out.println("   number of attributes: " + aNamedNodeMap.getLength()); 
		
		anAttribute = (Attr)aNamedNodeMap.item(i - 1);
		
		//Compare the attribute old value and replace it if needed
		if ( anAttribute.getValue().compareTo(pOldValue) == 0 )
		    anAttribute.setValue(pNewValue);
	    }
    }
    
    
    /*
     * replaceAttributeByValueRecursive(): recursively change the value of the attributes of a given element
     */
    public void replaceAttributeByValueRecursive(Element pElement, String pOldValue, String pNewValue)
    {
	NodeList aNodeList;
	int i;
	
	//###FC - DEBUG System.out.println("RR: old: " + pOldValue + " New: " + pNewValue);
	
	//First, deal with the current element's attributes
	replaceAttributeByValueInElement(pElement, pOldValue, pNewValue);
	
	//Recursively deal with the child elements of the current element
	aNodeList = pElement.getElementsByTagName("*"); //"*" is a special value that means "any tag" to getElementsByTagName
	
	for ( i = aNodeList.getLength(); i > 0; i-- )
	    replaceAttributeByValueRecursive((Element)aNodeList.item(i-1), pOldValue, pNewValue);
    }
    
    
    /*
     * replaceAttributeByValue(): change the value of the attribute pointed to by the given path
     */
    public void replaceAttributeByValue(String pPath, String pOldValue, String pNewValue)
    {
	StringTokenizer aTokenizer;
	Element anElement;
	NodeList someElements;
	String aString;
	
	
	//Initialization
	aTokenizer = new StringTokenizer(pPath, "/");
	
	if ( aTokenizer.countTokens() == 0 )
	    return;
	
	anElement = myDocument.getDocumentElement();
	aString = aTokenizer.nextToken();
	
	//Traverse the tree
	while ( aTokenizer.hasMoreTokens() )
	    {
		someElements = anElement.getElementsByTagName(aString);
		
		if ( someElements.getLength() != 1 )
		    {
			System.err.println("QoSINUS! wrong path traversed");
			return; //The path is wrong
		    }
		
		anElement = (Element)someElements.item(0);
		
		aString = aTokenizer.nextToken();
	    }
	
	//Replace the attribute's value starting at the current element, recursivly if requested
	if ( aString.compareTo("*") == 0 )
	    replaceAttributeByValueRecursive(anElement, pOldValue, pNewValue); //For us "*" also means recursive search
	else
	    replaceAttributeByValueInElement(anElement, pOldValue, pNewValue);			    
    }
}


/*****************************************************************************/
/* QaSXMLIf class:                                                           */
/*                 Interface QaS with the Xerces-J XML parser.               */
/*****************************************************************************/
class QaSXMLIf
{
    /*************************************************************************/
    /* Private                                                               */
    /*************************************************************************/
    
    private DOMParser myParser;
	
    
    /*************************************************************************/
    /* Public                                                                */
    /*************************************************************************/
    
    /*
     * Constructor: create the XML parser
     */
    public QaSXMLIf()
    {
	//Create parser
	try 
	    {
		myParser = new DOMParser();
	    }
	catch (Exception e) 
	    {
		System.err.println("error: Unable to instantiate parser "+e);
		return;
	    }
    }
    
    /*
     * parse(): parse a new request
     */
    public QaSXMLDoc parse(InputSource pSource)
    {
	Document aDOMDoc;   //The parsed DOM document
	QaSXMLDoc aQaSDoc;  //The QaS document object
	
	//Parse the SLS
	try 
	    {
		myParser.parse(pSource);
		aDOMDoc = myParser.getDocument();
            }
	catch (SAXParseException e) 
	    {
                // ignore
		return(null); //###FC - TBD: error checking
            }
	catch (Exception e) 
	    {
                System.err.println("error: Parse error occurred - "+e.getMessage());
                if (e instanceof SAXException) 
		    {
			e = ((SAXException)e).getException();
		    }
                e.printStackTrace(System.err);
		
		return(null); //###FC - TBD: error checking
            }
	
	aQaSDoc = new QaSXMLDoc(aDOMDoc);
	
	return(aQaSDoc);
    }
    
    
    /*
     * serialize(): serialize a document (no me digas...)
     */
    public byte[] serialize(QaSXMLDoc pDoc)
    {
	ByteArrayOutputStream aByteStream;
	OutputFormat aFormat;
	XMLSerializer aSerial;

	
	//Create the whole mess of objects...
	aByteStream = new ByteArrayOutputStream();
	aFormat  = new OutputFormat(pDoc.myDocument);
	
	try 
	    {
		aSerial = new XMLSerializer(aByteStream, aFormat);
		aSerial.asDOMSerializer();
		
		//..., serialize the document...
		aSerial.serialize(pDoc.myDocument.getDocumentElement());
	    } 
	catch ( Exception ex ) 
	    {
		ex.printStackTrace();
	    }
	
	//... and extract the resulting bytes
	return(aByteStream.toByteArray());
    }
}



