/*****************************************************************************/
/*                                                                           */
/*  qastbP.h                                                                 */
/*                                                                           */
/*                                                                           */
/*     Module:                                                               */
/*             qasmain.o module                                              */
/*                                                                           */
/*     Description:                                                          */
/*             Protected header file for the qastb.c module.                 */
/*                                                                           */
/*     History:                                                              */
/*             02/12/2002 - Fabien Chanussot - Creation                      */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
/* Use                                                                       */
/*****************************************************************************/

#include <net/pkt_sched.h>


/*****************************************************************************/
/* Macros                                                                    */
/*****************************************************************************/

/*****************************************************************************/
/* Types                                                                     */
/*****************************************************************************/

/* Token Bucket structure */
typedef struct qasTokenBucket_s qasTokenBucket_t;
struct qasTokenBucket_s 
{
  struct list_head linkedlist; //Linked list
  spinlock_t lock;             //List spinlock
  int flushing;                //Being flushed
  struct timer_list timer;     //Timer
  long timerperiod;            //Number of jiffies between two timer pulses
  u32 rate;                    //Bucket rate, in bytes per second
  u32 burst;                   //Bucket capacity, in usec
  u32 backlog;                 //Maximum number of bytes in the backlog
  u32 len;                     //Current number of bytes in the backlog
  psched_time_t timeref;       //Time of last call
  long tokens;                 //Tokens left over, in usec
  int resolution;              //Resolution in the rate evaluation, in bits to shift
  u32 rtab[256];               //Preevaluated rate table, in usec
};

/* Token Bucket entry structure */
typedef struct qasTokenBucketEntry_s qasTokenBucketEntry_t;
struct qasTokenBucketEntry_s 
{
    struct list_head list; //Linked list
    struct sk_buff *skb;   //Packet
    struct nf_info *info;  //Info field as passed by netfilter
};

/* Token bucket */
extern qasTokenBucket_t *qasTokenBucket;

/* token buckets table */
extern qasTokenBucket_t **qasdev_tbs;


/*****************************************************************************/
/* Public functions                                                          */
/*****************************************************************************/

qasTokenBucket_t *qasTokenBucketCreate(long rate, long burst, long backlog);
void qasTokenBucketChange(qasTokenBucket_t *tb, long rate, long burst);
void qasTokenBucketDelete(qasTokenBucket_t *tb);

extern int qasTokenBucketEnqueuePacket(struct sk_buff *skb, struct nf_info *info, void *data);
extern void qasTokenBucketDequeuePackets(qasTokenBucket_t *theTokenBucket, int pulse);
