/*****************************************************************************/
/*                                                                           */
/*  qos.h                                                                    */
/*                                                                           */
/*                                                                           */
/*     Module:                                                               */
/*             qos.so library                                                */
/*                                                                           */
/*     Description:                                                          */
/*             Public header file for the qos.c module.                      */
/*                                                                           */
/*     History:                                                              */
/*             31/08/2002 - Fabien Chanussot - Creation                      */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/* Use                                                                       */
/*****************************************************************************/

#include <errno.h>
#include <string.h>


/*****************************************************************************/
/* Macros                                                                    */
/*****************************************************************************/

/* Default datagram size used */
#define DEFAULT_DATAGRAM_SIZE 1500

/* Pues eso: display some debug information */
#define QoS_DEBUG 1

/* Premium DSCP = 46 (101110) */
#define QoS_PREMIUM 46

/* Error messages */
#ifdef QoS_DEBUG
#define QoS_M_error(num) \
  printf("Error %d at line %d, file %s\n", num, __LINE__, __FILE__)

#define QoS_M_errno(num) \
  printf("Error %d at line %d, file %s: %s\n", num, __LINE__, __FILE__, strerror(num))

#define QoS_M_errormsg(str) \
  printf("Error at line %d, file %s: %s\n", __LINE__, __FILE__, str)

#else
#define QoS_M_error(num) \
  return(0)
#define QoS_M_errno(num) \
  return(0)
#define QoS_M_errormsg(str) \
  return(0)
#endif


/*****************************************************************************/
/* Types                                                                     */
/*****************************************************************************/


/*****************************************************************************/
/* Public functions                                                          */
/*****************************************************************************/

//Simplified QoS specification
extern int QoS_Set(void *dest, int delai, int debit, int perte, char *QoSCode);

//QoS pseudo-negociation
extern int QoS_Negociate(void *dest, void *SLSin, void *SLSout);

//QoS session invocation
extern int QoS_Invoke(int sock, char QoSCode);

//QoS session termination
extern int QoS_Release(char QoSCode);
