///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  qos.hpp                                                                  //
//                                                                           //
//                                                                           //
//     Module:                                                               //
//             qos.so library                                                //
//                                                                           //
//     Description:                                                          //
//             QoS class definition                                          //
//                                                                           //
//     History:                                                              //
//             31/08/2002 - Fabien Chanussot - Creation                      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Use                                                                       //
///////////////////////////////////////////////////////////////////////////////

#include <errno.h>
#include <string.h>

#include <iostream>
using namespace std;


///////////////////////////////////////////////////////////////////////////////
// Macros                                                                    //
///////////////////////////////////////////////////////////////////////////////

// Qualitative values
#define QOS_HIGH   -1
#define QOS_MEDIUM -2
#define QOS_LOW    -3
#define QOS_ANY    -4

// Default datagram size used
#define DEFAULT_DATAGRAM_SIZE 1500

// Premium DSCP = 46 (101110)
#define QOS_PREMIUM 46

// Pues eso: display some debug information
#define QOS_DEBUG 1

// Error messages
#ifdef QOS_DEBUG
#define QoS_M_error(num) \
  printf("Error %d at line %d, file %s\n", num, __LINE__, __FILE__)

#define QoS_M_errno(num) \
  printf("Error %d at line %d, file %s: %s\n", num, __LINE__, __FILE__, strerror(num))

#define QoS_M_errormsg(str) \
  printf("Error at line %d, file %s: %s\n", __LINE__, __FILE__, str)
#else
#define QoS_M_error(num) \
  return(0)
#define QoS_M_errno(num) \
  return(0)
#define QoS_M_errormsg(str) \
  return(0)
#endif


///////////////////////////////////////////////////////////////////////////////
// QoS class                                                                 //
///////////////////////////////////////////////////////////////////////////////

class QoS
{
public:
  //Simplified QoS specification
  int Set(char *ipdest, char *portdest, int delay, int rate, int loss, const char *ipstreamdest, char *QoSCode);
  
  //QoS request
  int Request(char *ipdest, char *portdest, DOMDocument *SLSin, DOMDocument **SLSout);
  
  //QoS session invocation
  int Invoke(int sock, char QoSCode);
  
  //QoS session termination
  int Release(char *ipdest, char *portdest, char QoSCode);
};
