/*****************************************************************************/
/*                                                                           */
/*  qos.c                                                                    */
/*                                                                           */
/*                                                                           */
/*     Module:                                                               */
/*             qos.so library                                                */
/*                                                                           */
/*     Description:                                                          */
/*             Set of C wrapper functions that let applications specify      */
/*             their requirements and invoke QoS sessions.                   */
/*                                                                           */
/*     History:                                                              */
/*             30/08/2002 - Fabien Chanussot - Creation                      */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/* Use                                                                       */
/*****************************************************************************/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>

#include "qos.h"
#include "qosP.h"


/*****************************************************************************/
/* Macros                                                                    */
/*****************************************************************************/

#define CU_MASK 0x03

/*****************************************************************************/
/* Types                                                                     */
/*****************************************************************************/

/*****************************************************************************/
/* Private functions                                                         */
/*****************************************************************************/

/*****************************************************************************/
/* Protected functions                                                       */
/*****************************************************************************/

/*****************************************************************************/
/* Public functions                                                          */
/*****************************************************************************/

/*****************************************************************************/
/* Function: QoS_Set                                                         */
/* Role    : simplified request of QoS                                       */
/*****************************************************************************/
int QoS_Set(void *dest, int delai, int debit, int perte, char *QoSCode)
{
  QoS_M_errormsg("QoS_Set not implemented yet!!!");
  
  /* Done! */
  return(0);
}


/*****************************************************************************/
/* Function: QoS_Negociate                                                   */
/* Role    : request QoS from the system                                     */
/*****************************************************************************/
int QoS_Negociate(void *dest, void *SLSin, void *SLSout)
{
  QoS_M_errormsg("QoS_Negociate not implemented yet!!!");
  
  /* Done! */
  return(0);
}


/*****************************************************************************/
/* Function: QoS_Invoke                                                      */
/* Role    : program socket to mark DSCP field                               */
/*****************************************************************************/
int QoS_Invoke(int sock, char QoSCode)
{
  socklen_t len;
  char ctemp;
  
  
  /* The two last bits of the DSCP are reserved for future use: check that QoSCode is not too big... */
  if ( QoSCode > 63 )
    {
      QoS_M_error(QoSCode);
      return(-1);
    }
  
  /* Read the CU bits of the TOS field. */
  len = 1;
  if ( getsockopt(sock, IPPROTO_IP, IP_TOS, (void *)&(ctemp), &len) == -1 )
    {
      QoS_M_errno(errno);
      return(-1);
    }
  
  ctemp &= CU_MASK;
  
  /* Define the new TOS field  */
  QoSCode = (QoSCode << 2) | ctemp;
  
  /* Set the socket's IP_TOS option */
  if ( setsockopt(sock, IPPROTO_IP, IP_TOS, (void *)&QoSCode, 1) == -1 )
    {
      QoS_M_errno(errno);
      return(-1);
    }
  
  
  /* Done! */
  return(0);
}


/*****************************************************************************/
/* Function: QoS_Release                                                     */
/* Role    : release the DSCP assigned to the session                        */
/*****************************************************************************/
int QoS_Release(char QoSCode)
{
  QoS_M_errormsg("QoS_Release not implemented yet!!!");
  
  /* Done! */
  return(0);
}
